import React, { useState } from 'react'
import { useDrop } from 'react-dnd'
import { is, fromJS } from 'immutable'
import { Col } from 'antd'
import update from 'immutability-helper'
import Utils from '@/utils/utils.js'
import Card from './card'
import './index.scss'

const Container = ({list, setting, handleList, handleForm, closeForm, showField }) => {
  const [cards, setCards] = useState(list)
  const moveCard = (id, atIndex) => {
    const { card, index } = findCard(id)

    if (!card) return

    const _cards = update(cards, { $splice: [[index, 1], [atIndex, 0, card]] })

    handleList(_cards)
  }

  if (!is(fromJS(cards), fromJS(list))) {
    setCards(list)
  }

  const findCard = id => {
    const card = cards.filter(c => `${c.uuid}` === id)[0]
    return {
      card,
      index: cards.indexOf(card),
    }
  }

  const editCard = id => {
    const { card } = findCard(id)
    delete card.focus
    handleForm(card)
  }

  const closeCard = id => {
    const { card } = findCard(id)
    closeForm(card)
  }

  const setStyle = (style, id) => {
    const { card, index } = findCard(id)
    
    const _cards = update(cards, { $splice: [[index, 1, {...card, style}]] })

    handleList(_cards)
  }

  const copyCard = id => {
    const { card, index: overIndex } = findCard(id)

    let _card = fromJS(card).toJS()
    _card.uuid = Utils.getuuid()
    _card.focus = true

    // 复制到剪切板
    let oInput = document.createElement('input')
    let val = JSON.parse(JSON.stringify(_card))
    val.copyType = 'form'

    _card.$copy = true

    oInput.value = window.btoa(window.encodeURIComponent(JSON.stringify(val)))
    document.body.appendChild(oInput)
    oInput.select()
    document.execCommand('Copy')
    oInput.className = 'oInput'
    oInput.style.display = 'none'
    document.body.removeChild(oInput)

    const _cards = update(cards, { $splice: [[overIndex + 1, 0, _card]] })

    setCards(_cards)

    handleList(_cards, _card)
  }

  const [, drop] = useDrop({
    accept: 'form',
    drop(item) {
      if (item.hasOwnProperty('originalIndex')) {
        return
      }

      let newcard = {}
      newcard.uuid = Utils.getuuid()
      newcard.label = 'label'
      newcard.type = item.subType
      newcard.resourceType = '0'
      newcard.options = []
      newcard.readonly = 'false'
      newcard.required = 'true'
      newcard.span = 24
      newcard.focus = true

      if (item.subType === 'linkMain') {
        newcard.hidden = 'true'
        newcard.verifyVal = 'true'
      } else if (item.subType === 'textarea') {
        newcard.required = 'false'
      }

      let targetId = ''

      if (item.dropTargetId) {
        targetId = item.dropTargetId
        delete item.dropTargetId
      } else if (cards.length > 0) {
        targetId = cards[cards.length - 1].uuid
      }

      const { index: overIndex } = findCard(`${targetId}`) // cards为空时 overIndex 为 -1
      const _cards = update(cards, { $splice: [[overIndex + 1, 0, newcard]] })

      setCards(_cards)

      handleList(_cards, newcard)
    }
  })

  let style = {}
  if (setting.paddingLeft) {
    style.paddingLeft = setting.paddingLeft
  }
  if (setting.paddingRight) {
    style.paddingRight = setting.paddingRight
  }

  if (setting.display === 'drawer' && (setting.placement === 'top' || setting.placement === 'bottom') && setting.width !== 100) {
    style.minHeight = 'auto'
  }

  return (
    <div ref={drop} className="ant-row modal-fields-row mob-form" style={style}>
      {cards.map(card => {
        let span = card.span || 24
        if (![24, 12, 8, 6].includes(span)) {
          span = 24
        }
        return <Col key={card.uuid} span={span}>
          <Card
            id={card.uuid}
            key={card.uuid}
            card={card}
            showField={showField}
            moveCard={moveCard}
            editCard={editCard}
            setStyle={setStyle}
            closeCard={closeCard}
            copyCard={copyCard}
            findCard={findCard}
          />
        </Col>
      })}
    </div>
  )
}
export default Container
